
local CHUNK_SIZE = 32
local OFFSET_CHUNKS = 4
local BORDER_CHUNKS_OUTSIDE = 2
local BORDER_CHUNKS_INSIDE = 1
local WALL_TILES = 2
local WALL_MINABLE = false
local WALL_FORCE = "player"
local INSIDE_TREE_MULTIPLIER = 0.25
local SILO_POSITION = {x = 1520, y = 16} -- Should be in the middle of a chunk
local ENEMY_EXPANSION = false
local ORE_BASE_AMOUNT = 750
local OIL_BASE_AMOUNT = 60000 -- 20% yield, or 2 per second per well

local function ore_multiplier(x)
	local a = math.max(1, 2+x/(8*CHUNK_SIZE)) -- Ores get richer the further you are inside biter territory
	return a*a
end

script.on_init(function(event)
	game.forces["player"].chart(game.surfaces["nauvis"], {{SILO_POSITION.x-CHUNK_SIZE, SILO_POSITION.y-CHUNK_SIZE}, {SILO_POSITION.x+CHUNK_SIZE, SILO_POSITION.y+CHUNK_SIZE}})
	game.map_settings.enemy_expansion.enabled = ENEMY_EXPANSION
end)

script.on_event(defines.events.on_chunk_generated, function(event)
	local surface = event.surface
	if surface.name ~= "nauvis" then return end
	local area = event.area
	local x1 = area.left_top.x
	local x2 = area.right_bottom.x
	
	if area.right_bottom.x < CHUNK_SIZE*(OFFSET_CHUNKS-BORDER_CHUNKS_INSIDE)+1 then
		-- We are safely behind the wall. There are no biters here.
		for _, entity in pairs(surface.find_entities_filtered{area = area, force = "enemy"}) do
			entity.destroy()
		end
		-- There are also fewer trees.
		for _, tree in pairs(surface.find_entities_filtered{area = area, type = "tree"}) do
			local tree_filter = (global.tree_filter or 0) + INSIDE_TREE_MULTIPLIER
			if tree_filter > 1 then
				tree_filter = tree_filter - 1
			else
				tree.destroy()
			end
			global.tree_filter = tree_filter
		end
	elseif area.right_bottom.x < CHUNK_SIZE*OFFSET_CHUNKS+1 then
		-- We are in no man's land inside the wall. Everything has been razed.
		for _, entity in pairs(surface.find_entities_filtered{area = area}) do
			entity.destroy()
		end
		if area.right_bottom.x == CHUNK_SIZE*OFFSET_CHUNKS then
			-- Wall goes in this chunk
			for dy = 0,CHUNK_SIZE-1 do
				local py = area.left_top.y+dy+0.5
				for dx = 0,WALL_TILES-1 do
					local px = area.right_bottom.x-dx-0.5
					if surface.can_place_entity{name = "stone-wall", position = {px, py}, force = WALL_FORCE} then
						local e = surface.create_entity{name = "stone-wall", position = {px, py}, force = WALL_FORCE}
						e.minable = WALL_MINABLE
					end
				end
			end
		end
	elseif area.right_bottom.x < CHUNK_SIZE*(OFFSET_CHUNKS+BORDER_CHUNKS_OUTSIDE)+1 then
		-- We are in the safe zone outside the wall. There are no biters this close. Yet.
		for _, entity in pairs(surface.find_entities_filtered{area = area, force = "enemy"}) do
			entity.destroy()
		end
		-- There is also no ore. :(
		for _, entity in pairs(surface.find_entities_filtered{area = area, type = "resource"}) do
			entity.destroy()
		end
	end
	
	if SILO_POSITION.x > area.left_top.x and SILO_POSITION.x <= area.right_bottom.x and SILO_POSITION.y > area.left_top.y and SILO_POSITION.y <= area.right_bottom.y then
		for _, entity in pairs(surface.find_entities_filtered{area = {{SILO_POSITION.x-5, SILO_POSITION.y-6},{SILO_POSITION.x+6, SILO_POSITION.y+6}}}) do
			entity.destroy()
		end
		local tiles = {}
		local i = 1
		for dx = -6,6 do
			for dy = -7,6 do
				tiles[i] = {name = "grass-1", position = {SILO_POSITION.x+dx, SILO_POSITION.y+dy}}
				i=i+1
			end
		end
		surface.set_tiles(tiles, false)
		tiles = {}
		i = 1
		for dx = -5,5 do
			for dy = -6,5 do
				tiles[i] = {name = "concrete", position = {SILO_POSITION.x+dx, SILO_POSITION.y+dy}}
				i=i+1
			end
		end
		surface.set_tiles(tiles, true)
		local silo = surface.create_entity{name = "rocket-silo", position = {SILO_POSITION.x+0.5, SILO_POSITION.y}, force = "player"}
		silo.destructible = false
		silo.minable = false
	end
	
	-- Adjust ore richness
	-- This is calibrated only for vanilla.
	for _, resource in pairs(surface.find_entities_filtered{area = area, type="resource"}) do
		if resource.prototype.resource_category == "basic-fluid" then
			local base_amount = OIL_BASE_AMOUNT
			resource.amount = base_amount * ore_multiplier(resource.position.x)
		elseif resource.prototype.resource_category == "basic-solid" then
			local base_amount = ORE_BASE_AMOUNT
			resource.amount = base_amount * ore_multiplier(resource.position.x)
		end
	end
end)

script.on_event(defines.events.on_research_finished, function(event)
	local recipes = event.research.force.recipes
	-- Having an entire rocket silo in your pocket? What a ridiculous notion. Bonkers, I say. Things don't fit in there.
	if recipes["rocket-silo"] then recipes["rocket-silo"].enabled = false end
end)



